@@ -75,7 +75,6 @@ int main( int argc, char* argv[] )
     ( "help", "produce help message" )
     ( "progress", "display progress" )
     ( "verbose",  "verbose mode" )
-    ( "novalidate",  "remove validation of input geometries" )
     ( "vtk",  "output vtk geom on failure" )
     ;
 
@@ -92,9 +91,6 @@ int main( int argc, char* argv[] )
     bool progress = vm.count( "progress" ) != 0 ;
     bool vtk = vm.count( "vtk" ) != 0 ;
 
-    SFCGAL::algorithm::SKIP_GEOM_VALIDATION = vm.count( "novalidate" ) != 0 ;
-
-
     //boost::timer timer ;
     boost::chrono::system_clock::time_point start = boost::chrono::system_clock::now();
 
@@ -428,6 +424,7 @@ int main( int argc, char* argv[] )
         }
 
         CATCH_INVALID_GEOM_AND_NOT_IMPLEMENTED( ( void )algorithm::straightSkeleton( *geom1 ) ; )
+        CATCH_INVALID_GEOM_AND_NOT_IMPLEMENTED( ( void )algorithm::straightSkeleton( *geom1, /*autoOrientation*/ false, /*innerOnly*/ true, /*outputDistanceInM*/ true ) ; )
         CATCH_INVALID_GEOM_AND_NOT_IMPLEMENTED( ( void )algorithm::tesselate( *geom1 ) ; )
 
         CATCH_INVALID_GEOM_AND_NOT_IMPLEMENTED(
